"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Mar 8, 2010

@author: Martin Holst Swende
'''
from PyQt4.QtCore import * 
import aggregate

def to_number(s):
    try:
        return int(s)
    except ValueError:
        return False

class TreeItem():
    """This is a data wrapper to model tree items in a view. This class is
    losely based on the QT example at http://doc.qt.nokia.com/4.6/itemviews-simpletreemodel.html """
    
    def __init__(self,parent=None, data=None, label=None):
        self.childDataCount = 0
        self.columnLabels = [label, ""]
        self.children = []
        self.parent = parent
        if data != None : 
            if type(data).__name__=='dict':
                for k,v in data.items() :
                        self.addChild(TreeItem(self,data=v,label=k))
                self.columnLabels[1] = self.columnLabels[1]  + ("(%s)" % self.childDataCount)
            elif type(data).__name__=='list':
                for l in data:
                    self.addChild(TreeItem(self,data=l))
                    self.columnLabels[1] = self.columnLabels[1]  + ("(%s)" % self.childDataCount)
            else:
                if label != None:
                    try:
                        self.childDataCount= int(data)
                    except ValueError:
                        pass
                    self.columnLabels=[label, data]
                else: 
                    self.columnLabels[0] = data;
    def childCount(self):
        """returns the total number of children"""
        return len(self.children)
    def childNumber(self):
        """The childNumber() function is used to determine the index of the child in its parent's list of children"""
    def columnCount(self):
        """Gets the number  of columns"""
        return len(self.columnLabels)
    def row(self):
        if(self.parent == None):
            return 0
        #Find this item in parent children
        row =0
        for child in self.parent.children:
            if child == self:
                return row
            row = row+1
    def data(self, column):
        """Returns the data for the specified column"""
        return self.columnLabels[column]
    def addChild(self, child):
        """Adds a child to this node"""
        self.children.append(child)
        self.childDataCount = self.childDataCount+child.childDataCount
        
    def out(self, tab=0):
        print "-"*tab+str(self.columnLabels)
        for child in self.children:
            child.out(tab+1) 

                
    @staticmethod
    def createTree(data, key, count):
        """Creates a tree based on a list of dictionarys and labels for the keys. The list can typically look like this : 
        [
        {
        "request.headers.Host" : "dn.se",
        "count" : {
            "Apache" : 1
        }
        },
        {
        "request.headers.Host" : "www.dn.se",
        "count" : {
            "Apache" : 133,
            "undefined" : 7
        }
        }
        ]
        If this is used with labels {"request.headers.Host":"Host"},
        This would generate the following tree : 
        ['root']
        -['dn.se']
        --['Apache:1']
        -['www.dn.se']
        --['Apache:133']
        --['undefined:7']
        """
        topnode = TreeItem(None)
        topnode.setData(["root","test"])
        for group in data:
            #This level should be dicts
            label =group[key]
            groupNode = TreeItem(topnode)
            topnode.addChild(groupNode)
            num =0
            for k,v in group[count].items():
                num = num +1
#                countItem = TreeItem(groupNode, "%s:%s" % (str(k), str(v)))
                countItem = TreeItem(groupNode, [str(k), str(v)])
                groupNode.addChild(countItem)
            groupNode.setData([label,"%d requests" % num])
        topnode.out()
        return topnode

class QMongoTreeModel(QAbstractItemModel):
    
    '''
    This is the data model which provides the table tree view of aggregate data. 
    
    When subclassing QAbstractItemModel, at the very least you must implement 
    index(), parent(), rowCount(), columnCount(), and data(). These functions are used
     in all read-only models, and form the basis of editable models.

    You can also reimplement hasChildren() to provide special behavior for models where the implementation of rowCount() is expensive. This makes it possible for models to restrict 
    the amount of data requested by views, and can be used as a way to implement lazy population of model data.

    http://www.riverbankcomputing.co.uk/static/Docs/PyQt4/html/qabstractitemmodel.html#details
    '''
    # Some default values
    
    def __init__(self, conn): 
        QAbstractItemModel.__init__(self) 
        self.conn = conn
        self.aggregator = aggregate.DBAggregator.getAggregators()[0]
        self.loadData()

    def reload(self):
        self.loadData()
        
    def setAggregator(self, aggregator):
        self.aggregator = aggregator
    def getAggregator(self):
        return self.aggregator
    
    def loadData(self, limit=100):
        """ 
        Loads data from the database, and applies transformations
        Group aggregate data db-calls typically looks like this : 
        db.conversations.group({cond:{},key:{"request.url.path":1},initial:{count:{}},$reduce:function(obj,prev){var s=obj.response.status;prev.count[s]=prev.count[s]?prev.count[s]+1:1;}})

        http://www.mongodb.org/display/DOCS/Aggregation

        Python API : http://api.mongodb.org/python/1.5.1%2B/api/pymongo/collection.html
        group(key, condition, initial, reduce, finalize=None, command=True)
        Perform a query similar to an SQL group by operation.
        Returns an array of grouped items.
        The key parameter can be:
                * None to use the entire document as a key.
                * A list of keys (each a basestring) to group by.
                * A basestring or Code instance containing a JavaScript function to be applied to each document, returning the key to group by.
        Parameters:    
            * key: fields to group by (see above description)
            * condition: specification of rows to be considered (as a find() query specification)
            * initial: initial value of the aggregation counter object
            * reduce: aggregation function as a JavaScript string
            * finalize: function to be called on each object in output list.
            * command (optional): DEPRECATED if True, run the group as a command instead of in an eval - 
            this option is deprecated and will be removed in favor of running all groups as commands
        """
        query = self.aggregator
        #print ( query )
        items = self.conn.collection.group(key=query.key(), condition=query.cond(), initial=query.initial(), reduce=query.reduce())
        #print(items)
        self._root = TreeItem (parent = None, label='root')
        key = query.key()[0]
        for item in items:
            header = item[key]
            #print header
            count = item['count']
            #print count
            treeItem = TreeItem(parent=self._root, label=header, data=count)
            self._root.addChild(treeItem)
            
        #self._root.out()
    def index(self, row, column, parentIndex):
        if not self.hasIndex(row, column, parentIndex):
            return QModelIndex()
        
        parentItem = self._root
        if parentIndex.isValid():
            parentItem = parentIndex.internalPointer()

        child = parentItem.children[row]
        if child != None:
            return self.createIndex(row, column, child);
        return QModelIndex()
    def parent(self, index):
        
        if not index.isValid():
            return QModelIndex()
        
        child = index.internalPointer()
        parent = child.parent;
        
        if(parent== self._root):
            return QModelIndex()
        
        return self.createIndex(parent.row(), 0, parent);
    
    def _rowCount(self,parentIndex):
        val = self._rowCount(parentIndex)
        print("rowcount (%s,%s) returning %s" %(str(parentIndex.row()),str( parentIndex.column()), str(val)))
        return val
    def rowCount(self,parentIndex):
        if parentIndex.column() > 0:
                return 0
            
        parentItem = self._root
        if parentIndex.isValid():
            parentItem = parentIndex.internalPointer()
            
        return parentItem.childCount()
    
    def _columnCount(self, parentIndex):
        val = self._columnCount(parentIndex)
        #print("columnCount (%s,%s) returning %s" %(str(parentIndex.row()),str( parentIndex.column()), str(val)))
        return val
    def columnCount(self, parentIndex):
        if not parentIndex.isValid():
            return self._root.columnCount()
        return parentIndex.internalPointer().columnCount()
    
    def _data(self, index, role): 
        val = self._data(index,role)
        #print("data returning "+str(val))
        return val 
    def data(self, index, role):
        if not index.isValid(): 
            return QVariant() 
        if role != Qt.DisplayRole: 
            return QVariant()
        
        item = index.internalPointer()
        return item.data(index.column()) 
    
    def headerData(self, index, orientation, role):
        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
                return QVariant("Heppa!")
            
        return QVariant("row %d" % index)
    
