"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Mar 21, 2010

@author: Martin Holst Swende
'''
from hatcore.prefecences import prefs
    
    
class DBAggregator():
    
    def __init__(self, cond={}, initial={"count":{}}, key=[], reduce="", name="Foo?", reducejsfile=None, help = "No documentation added"):
        self._cond = cond
        self._initial = initial
        self._key = key
        self._reduce = reduce
        # Please override this...
        self._name = name
        if reducejsfile is not None:
            self._reduce = self.loadJS(reducejsfile)
        self.__help = help
        
    def loadJS(self, filename):
        file = open("%s/javascript/%s" % (prefs.getBasedir(), filename), 'r');
        data = file.read()
        return data

    def cond(self):
        """ Returns the condition (db filter) this aggregator uses"""
        return self._cond
    
    def key(self):
        return self._key
    
    def initial(self):
        return self._initial
    
    def reduce(self):
        return self._reduce
    def name(self):
        return self._name
    def getHelp(self):
        return self.__help
_aggregators = [
        DBAggregator(name="AggregatePaths", key=['request.headers.Host'], reducejsfile='aggregate_paths.js',
                     help="Aggregates data by sorting all conversations into a tree-view of the directory. Shows statuscode and count."),
        DBAggregator(name="AggregatePathsSimple", key=['request.url.path'], reducejsfile='aggregate_path_simple.js',
                     help="Aggregates data by grouping requests according to unique paths"),
        DBAggregator(name="HTTP Status -> path", key=['response.status'], reducejsfile='aggregate_path_simple.js', 
                     help="Aggregates data by grouping urls according to response status"),
        DBAggregator(name="Host->Server banner", key=['request.headers.Host'], reducejsfile='aggregate_serverheader.js',
                    help="Aggregates data by grouping server header according to response status"),
        DBAggregator(name="List response headers", key=['1'], reducejsfile='aggregate_server_headers.js', 
                     help="Aggregates all response headers under a root item"),
        DBAggregator(name="Host -> Parameter names", key=['request.headers.Host'], reducejsfile='aggregate_parameternames.js', 
                     help="Aggregates parameter names per host"),
        DBAggregator(name="Host->Parameter name->value", key=['request.headers.Host'], reducejsfile='aggregate_parameternames_and_values.js', 
                     help="Aggregates parameter names and values by host"),
                ]
def getAggregators():
    """
    Convenience factory method to get all aggregator functions
    """
    return _aggregators        


    
