"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import pymongo
'''
Created on Feb 21, 2011

@author: martin
'''

class Querier(object):
    
    '''
    This class takes care of querying the database. Different UI components may set different values,e.g the query specification
    or the query fields. These are remembered and used once a query is actually performed
    '''
    def __init__(self, dbconn, listener):
        '''
        Constructor
        '''
        self.conn = dbconn
        # There may be listeners
        self.listeners = [listener]
        
        #Set default values
        self.set(spec = None, limit=0, skip=0, sort="request.time", direction=pymongo.DESCENDING)
        
    def set(self,spec=None, limit = 0, skip=0, sort=None, direction=None ):
        self.spec = spec
        self.limit = limit
        self.skip = skip
        self.sort = sort
        self.direction = direction
        if sort is not None:
            self.__sort=[(sort, direction)]
        else: 
            self.__sort = None
            
    def find(self, fields):
        return self.conn.collection.find(spec = self.spec,fields=fields , limit = self.limit, skip = self.skip, sort=self.__sort)
    
    def test(self, spec=None):
        return self.conn.collection.find(spec = spec).count()
    
    def loadById(self, id):
        return self.conn.collection.find_one(id)
    
    def apply(self):
        for listener in self.listeners:
            listener.onQueryChanged()