"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Sep 7, 2010

@author: martin
'''
import os
import pickle
class Prefs():
    """Class that holds the user preferences"""
    
    def __init__(self):
        self._prefs = {}
        self.HOME = os.getenv("HOME")

    def setBasedir(self, basedir):
        self.basedir = basedir
        
    def getBasedir(self):
        return self.basedir
    
    def setPreference(self, key, value):
        """Sets a preference to be stored. Any component
        can use the preference object and store arbitrary
        values here.
        
        Any pickable object can be stored, i.e not only strings. 
         
        Example: prefs.setPreference("aggregator_windowsize", [120,120,230,230])
        @param key - the key to use for lookup
        @param value - the actual value to store
        """
        self._prefs[key] = value
        
    def getPreference(self, key):
        """Fetch the preference for a specific key. Returns None
        if no such key is stored"""
        if key in self._prefs:
            return self._prefs[key]
        return None

    def load(self):
        """Loads preferences from file"""
        try:
            f = open("%s/.hatkit_preferences" % self.HOME,"r")
            preferences = pickle.load(f)
            f.close()
            self._prefs = preferences
        except IOError:
            print ("Could not find preferences-file")
            self._prefs = {}
            return
        
    def save(self):
        """Saves preferences"""
        try:
            f = open("%s/.hatkit_preferences" % self.HOME,"w")
            pickle.dump(self._prefs, f)
            f.close()
        except IOError as ioe:
            print ioe
            print ("Could not find preferences-file, creating it now")
        
        
#The global preferences
prefs = Prefs()
