"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Mar 26, 2010

@author: martin
'''
from hatcore.DBConnector import MongoConnector

from core.data.url.httpResponse import httpResponse
from core.data.dc.dataContainer import dataContainer
from core.data.kb.knowledgeBase import knowledgeBase
from plugins import grep
import sys

class Analyser(MongoConnector):
    '''
    This is the base class for third-party analysers. Such analysers can be implemented to 
    e.g route requests/responses through Rat-proxy or the grep-plugins from w3af. 
    
    When subclassing this class, you can do one of two things : 
    - Implement analyse_objects(request, response) and analyse the data in object-form
        OR
    - Implement analyse_raw(request, response) and anlyse the data in rawest possible form. This is suitable 
    for using with existing analysers (e.g rats) which operates on raw data. 
    '''


    def __init__(self):
        '''
        Constructor
        '''
        self.setup()
    
    def handleData(self):
        pass
    def start(self, spec=None, limit=100):
        """
        This starts the analysation process. This method will run through all requests/responses and call analyse 
        with them. 
        @param spec - The mongodb filter specifying what conversations to load
        @param limit - the limit to how many to run through
        """
        use_raw = hasattr(self, "analyse_raw")
        
        if not use_raw and not hasattr(self,"analyse_objects"):
            print "Error, analysers must implement either analyse_raw or analyse_objects"
            return
        
        fields = ["request", "response","request-raw", "response-raw"]
        
        if use_raw:
            method = self.analyse_raw
        else:
            method = self.analyse_objects 
            
        for item in self.collection.find(fields=fields, limit=limit):
            request = item[fields[0]]
            response = item[fields[1]]
            request_raw = item[fields[2]]
            response_raw = item[fields[3]]
            data = method(request, response, request_raw, response_raw)
            print data
            
