"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

'''
Created on Apr 5, 2010

@author: Martin Holst Swende
'''
import thirdparty
from core.data.kb.knowledgeBase import knowledgeBase, kb
from plugins.grep.findComments import findComments
from core.data.url.httpResponse import httpResponse
from core.data.dc.dataContainer import dataContainer
import sys
import os
import glob
from core.controllers.misc import temp_dir
import re

W3AF_PATH = "/home/martin/workspace/w3af/1.0"
class RequestWrapper():
    def __init__(self, request):
        self.request = request;
        self.dc = dataContainer()
    def getDc(self):
        return self.dc
    def getBody(self):
        return self.request["content"]
    def getMethod(self):
        return self.request["method"]
    def getURI(self):
        return "http://"+self.request["url"]["raw"]
    def getURL(self):
        return "http://"+self.request["url"]["path"]
    def getData(self):
        if "raw" in self.request.keys():
            return self.request.keys["raw"]
        return ""
    
class W3afGrepper(thirdparty.Analyser):
    from plugins.grep.ajax import ajax
    from plugins.grep.blankBody import blankBody
    from plugins.grep.domXss import domXss
    from plugins.grep.findComments import findComments
    def __init__(self):
        thirdparty.Analyser.__init__(self)
        def monkey_append( self, callingInstance, variableName, value ):
            '''
            This is a method used to monkey-patch the w3af knowledgebase
            '''
            if isinstance( callingInstance, basestring ):
                name = callingInstance
            else:
                name = callingInstance.getName()
            print("[%s] - [%s]:"% (name, variableName))
            print(value)
            
        def monkey_is404( httpResponse ):
            
            """This is a method used to monkey-patch the w3af knowledgebase is404"""
            if httpResponse.getCode() == 404:
                return True
            return False
        
        knowledgeBase.append = monkey_append
        kb.save( 'error404page', '404', monkey_is404)
        self.greppers = self.createGreppers()
        
    def analyse_objects(self, req, res, request_raw, response_raw):
        #grepper = domXss()
        
        url =  "http://"+req["headers"]["Host"]+req["url"]["path"]
        #Need to convert all headers to utf8 instead of unicode
        utf8Hdrs = {}
        for hdr, val in res["headers"].items():
            utf8Hdrs[hdr.encode("utf-8")] = val.encode("utf-8")
        
        w3afresponse = httpResponse(int(res["status"]), res["raw"],utf8Hdrs, url, url, res["reason"]
                     , 123, res["rtt"])
        from time import time
        for grepper in self.greppers:
            grepper.grep(RequestWrapper(req), w3afresponse)
#            print("Running grepper: %s:" % grepper.getName())
    
        print "Done"
    def createInstance(self,moduleName, *args):
        # Import the module
        try:
            print("Importing %s" % moduleName)
            __import__(moduleName)
        except ImportError,  ie:
            raise Exception('Error while loading module "'+ moduleName + '". Exception: ' + str(ie) )
        # Load the class
        try:
            aModule = sys.modules[moduleName]
            className = moduleName[moduleName.rfind(".")+1:]
            aClass = getattr( aModule , className )
        except:
            raise Exception('Error loading the module ("'+ moduleName + '") .')
        #Instantiate an object of said class
        try:
            res = aClass(*args)
            return res
        except Exception, e:
            raise Exception('Failed to get an instance of "%s". Original exception : "%s"' % ( className , str(e)))


    def createGreppers( self ):
        # Some plugins need a temp dir to store stuff
        temp_dir.create_temp_dir()
        w3afGreppers = []
        path = W3AF_PATH+"/plugins/grep/*.py"
        greppers = glob.iglob(path)
        print "Path: %s " % path
        for file in greppers:
            file = file.split("/")[-1]
            moduleName = file[:-3]
            if moduleName == "__init__":
                continue
            #print "Grep plugin : %s" % moduleName
            plugin =self.createInstance("plugins.grep.%s" % moduleName)
        
            
            # All plugins with no dependencies are inserted first
            # then all plugins with dependencies which are grep-type
            # Hopefully, the depended upon plugins will be placed prior
            # This will not work if dependencies are more complex, but
            # currently, there is only one plugin which depends on another
            if len(plugin.getPluginDeps()) == 0 :
                w3afGreppers.insert(0, plugin)
            else:
                #Test that there are no non-grep dependencies
                for dep in plugin.getPluginDeps():
                    depType, depPlugin = dep.split('.')
                    if depType == "grep":
                        w3afGreppers.append(plugin)
                    else:
                        print "Failed to load dependence %s for plugin %s" % (depPlugin, moduleName)
        return w3afGreppers

def testInstantiation():
    w3af = W3afGrepper()
    for item in w3af.greppers : 
        print item
def testRegexp():
        test = "        date: 2003/11/04 16:42:00;  author: davem;  state: Exp;  lines: +0 -58"
        my  = "date:.*author:\W(\w+);"
        
        # Add the regex
        
        regex = '\$.*?: .*? .*? \d{4}[-/]\d{1,2}[-/]\d{1,2}'
        regex += ' \d{1,2}:\d{1,2}:\d{1,2}.*? (.*?) (Exp )?\$'
        cregex = re.compile(regex)
        cmy = re.compile(my)
        for m in cregex.findall(test):
            print m
        for m in cmy.findall(test):
            print m

if __name__ == '__main__':
#    testInstantiation()
    W3afGrepper().start()
    #testRegexp()