"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Feb 12, 2011

@author: martin
'''
from ui_generated import aggregator_settings
import glob
from aggregate import DBAggregator
from PyQt4 import QtCore, QtGui
from ui_generated import aggregator
import aggregate.AggregationModels
from ui.WindowHandling import WindowUtils
from hatcore.prefecences import prefs

class AggregationWindow(aggregator.Ui_Form):
    
    def __init__(self, conn):
        self.model = aggregate.AggregationModels.QMongoTreeModel(conn)

    def setupUi(self, window):
        self.window = window
        aggregator.Ui_Form.setupUi(self, window)
        self.treeView.setModel(self.model)
        self.pushButton_setup.clicked.connect(self.showSettings)
        self.pushButton_expandall.clicked.connect(self.treeView.expandAll)

    def showSettings(self):
        """Shows the edit-form for managing the aggregator setup"""
        print("showSettings called")
        window = QtGui.QWidget()
        ui = AggregationSettingsWindow(self.model, self.treeView)
        WindowUtils.display(window, ui)

class AggregationSettingsWindow(aggregator_settings.Ui_Form):

    def __init__(self, treeModel, treeView):
        self.treeModel = treeModel
        self.treeView = treeView
        self.reduceFiles = glob.glob(prefs.getBasedir()+"/javascript/*.js")
        self.predefinedAggregators = aggregate.DBAggregator.getAggregators()

    def setupUi(self, DBDialog):
        super(AggregationSettingsWindow, self).setupUi(DBDialog)
        #Setup pre-loaded aggregators
        self.pushButton_apply.clicked.connect(self.apply)
        self.pushButton_help.clicked.connect(self.showHelp)
        self.pushButton_revert.clicked.connect(self.revert)
        #TODO! Below gives error, currentIndexChanged(int) is used

        QtCore.QObject.connect(self.comboBox_reduce, QtCore.SIGNAL("currentIndexChanged(QString)"), self.loadReduceFunction)
        QtCore.QObject.connect(self.comboBox_predefined, QtCore.SIGNAL("currentIndexChanged(QString)"), self.loadAggregator)

        self.infoHtml = str(self.textBrowser_info.toHtml())

        for a in self.predefinedAggregators:
            self.comboBox_predefined.addItem(a.name(),a)
        for x in self.reduceFiles:
            self.comboBox_reduce.addItem(x)
        #Preload with current values
        self.revert()
    
    def showHelp(self):
        print ("Not implemented yet")
    
    def loadReduceFunction(self, data):
        """Loads a function (js) from file"""
        f = open(str(data), 'r')
        js = f.read()
        self.textEdit_reduce.setText(js)
        
    def loadAggregator(self, data, aggregator=None):
        index = self.comboBox_predefined.currentIndex()
        aggregator = self.comboBox_predefined.itemData(index).toPyObject()
        #Replace the #info# with text
        html = self.infoHtml.replace("#info#",aggregator.getHelp())
        self.textBrowser_info.setHtml(html)
        self.setAggregator(aggregator)
        
    def setAggregator(self,a):
        cond = a.cond()
        initial = a.initial()
        reduce = a.reduce()
        key = a.key()
        
        #populate ui
        self.lineEdit_cond.setText(str(cond))
        self.lineEdit_initial.setText(str(initial))
        self.lineEdit_key.setText(str(key))
        self.textEdit_reduce.setText(str(reduce))
        
    def revert(self):

        #Get currently used values
        a = self.treeModel.getAggregator()
        self.setAggregator(a)
        
    def apply(self):
        #Get input
        cond = str(self.lineEdit_cond.text())
        initial = str(self.lineEdit_initial.text())
        reduce = str(self.textEdit_reduce.toPlainText())
        key = str(self.lineEdit_key.text())

        #Test if it is ok
        try:
            cond = eval(cond)
            initial = eval(initial)
            key = eval(key)
        except Exception, e:
            print(e)
            return
        #Construct aggregator
        aggregator = DBAggregator.DBAggregator(cond=cond, initial=initial, reduce=reduce, key=key)
        #Modify the model
        self.treeModel.setAggregator(aggregator)
        self.treeModel.reload()
        #Update the view, by setting 'new' model via None
        self.treeView.setModel(None)
        self.treeView.setModel(self.treeModel)
        print("Aggregation applied")
