"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Feb 20, 2011

@author: martin
'''
from ui_generated.tabledata_settings import Ui_Form
from PyQt4 import QtGui
import ast
from ui.UIHelpers import ComboboxSettingsHandler
from traceback import format_exc
import pymongo

class DatabaseFilterHandler(object):
    '''
    This class handles the query filters
    '''
    def __init__(self, form, querier):
        '''
        Constructor
        '''
        #form = Ui_Form() 
        # Connect handlers
        self.querier =  querier
        
        #self.pushButton_loadjs = form.pushButton_loadjs
        #form.pushButton_load_native.clicked.connect(self.onLoadNative)
        #self.comboBox_native = form.comboBox_native
        form.pushButton_loadexternal_js.clicked.connect(self.onLoadExternalJs)  
        #form.pushButton_save_native.clicked.connect(self.onSaveNative)
        #form.pushButton_savejs.clicked.connect(self.onSaveJs)
        self.lineEdit_native_expression = form.lineEdit_native_expression
        #self.lineEdit_savename_javascript = form.lineEdit_savename_javascript
        #self.lineEdit_savename_native = form.lineEdit_savename_native
        self.plainTextEdit_javascript = form.plainTextEdit_javascript
      
        self.lineEdit_limit = form.lineEdit_limit
        self.lineEdit_skip = form.lineEdit_skip
        self.lineEdit_sort = form.lineEdit_sort
        self.checkBox_sortdirection = form.checkBox_sortdirection
        
        self.plainTextEdit_output_js = form.plainTextEdit_output_js
        self.plainTextEdit_output_native = form.plainTextEdit_output_native
      
        form.pushButton_apply_js_filter.clicked.connect(self.apply_jsfilter)
        form.pushButton_help_js_filter.clicked.connect(self.showHelp_jsfilter)

        form.pushButton_apply_native_filter.clicked.connect(self.apply_nativefilter)
        form.pushButton_help_native_filter.clicked.connect(self.showHelp_nativefilter)
      
        form.pushButton_test_js.clicked.connect(self.test_jsfilter)
        form.pushButton_test_native.clicked.connect(self.test_nativefilter)
        
        form.pushButton_revert_native_filter.clicked.connect(self.revert)
        
        self.nativeSettingsHandler = ComboboxSettingsHandler(form.comboBox_native, form.pushButton_load_native, form.pushButton_save_native, form.lineEdit_savename_native, self, "native-settings")
        self.jsSettingsHandler = ComboboxSettingsHandler(form.comboBox_js,  form.pushButton_loadjs,form.pushButton_savejs, form.lineEdit_savename_javascript, self, "js-settings")
        # Set the currently used setting
        self.revert()
        
        
    def revert(self):
        """Discards current changes and reloads the configuration from the querier object"""
        self.lineEdit_limit.setText(str(self.querier.limit))
        self.lineEdit_skip.setText(str(self.querier.skip))
        self.lineEdit_sort.setText(str(self.querier.sort or ""))
        self.lineEdit_native_expression.setText(str(self.querier.spec or ""))

    def onLoadExternalJs(self):
        """Called when user wants to load an external js - file """
        fileName = QtGui.QFileDialog.getOpenFileName(None, "Open Javascript file", "", "Javascript files (*.js)");
        print("Loading file %s " % fileName)
        file = open(fileName, 'r');
        content = file.read()
        self.plainTextEdit_javascript.setPlainText(content)

    def getSaveData(self, name):
        """Returns the data which should be persisted if the user presses 'save' the current setup"""
        if name == "js-settings":
            return self.plainTextEdit_javascript.toPlainText()
        elif name == "native-settings" : 
            return self.lineEdit_native_expression.text()
    
    def applySaveData(self, data, name):
        """Applies whatever was saved in getSaveData"""
        if name == "js-settings":
            self.plainTextEdit_javascript.setPlainText(data)
        elif name == "native-settings" : 
            self.lineEdit_native_expression.setText(data)
    
    def __getLimit(self):
        """
        @return either the integer value for limit or 0
        """
        x = str(self.lineEdit_limit.text())
        if len(x) == 0 : 
            return 0
        return int(x)
    
    def __getSkip(self):
        """
        @return either the integer value for skip or 0
        """
        x = str(self.lineEdit_skip.text())
        if len(x) == 0 : 
            return 0
        return int(x)
    def __getSort(self):
        x = str(self.lineEdit_sort.text())
        if len(x) == 0 : 
            return None
        return x
    
    def __getDirection(self):
        if self.checkBox_sortdirection.isChecked():
            return pymongo.ASCENDING
        return pymongo.DESCENDING
    
    def test_jsfilter(self):
        """Test the current js-expression"""
        exp = str(self.plainTextEdit_javascript.toPlainText())
        object = {"$where" : exp}
        self.__test(object, self.plainTextEdit_output_js.setPlainText)
        
    def test_nativefilter(self):    
        exp = str(self.lineEdit_native_expression.text())
        output = self.plainTextEdit_output_native.setPlainText
        try:
                object = None
                if len(exp) > 0 : 
                    object = ast.literal_eval(exp)
                self.__test(object,output )
        except Exception as e:
            print(e)
            output(format_exc( 1))

    def __test(self, spec, output):
        """Performs a test, basically only performs a query and .count() on the resulting cursor.
        @param spec - the specification to use
        @param output - a method which is called with content to display, either count-information or error information
        """
        try:
            count = self.querier.test(spec=spec )
            output("Count: %i" % int(count))
        except Exception as e:
            print(e)
            output(format_exc( 1))

    def apply_jsfilter(self):
        """Applies the javascript filter"""
        exp = str(self.plainTextEdit_javascript.toPlainText())
        self.querier.set(spec={"$where" : exp}, limit= self.__getLimit(),skip = self.__getSkip() , sort=self.__getSort(), direction=self.__getDirection())
        self.querier.apply()
        
    def apply_nativefilter(self):
        """Applies the native filter"""
        exp = str(self.lineEdit_native_expression.text()) 
        object = None
        if len(exp) > 0 : 
            object = ast.literal_eval(exp)
        self.querier.set(spec =object, limit= self.__getLimit(),skip = self.__getSkip() , sort=self.__getSort(), direction=self.__getDirection())
        self.querier.apply()

        
    def showHelp_jsfilter(self):
        """
        From: http://www.mongodb.org/display/DOCS/Advanced+Queries#AdvancedQueries-JavascriptExpressionsand%7B%7B%24where%7D%7D : 
         To do so, pass a string containing a Javascript expression to find(), or assign such a string to the query object member $where. 
        In addition to the structured query syntax shown so far, you may specify query expressions as Javascript.
        The database will evaluate this expression for each object scanned. When the result is true, the object is returned in the query results.

        For example, the following statements all do the same thing:

        db.myCollection.find( { a : { $gt: 3 } } );
        db.myCollection.find( { $where: "this.a > 3" } );
        db.myCollection.find("this.a > 3");
        f = function() { return this.a > 3; } db.myCollection.find(f);
        Javascript executes more slowly than the native operators listed on this page, but is very flexible. See the server-side processing page for more information.
        """
        print("showHelp_jsfilter")
        
    def showHelp_nativefilter(self):
        """
        @see http://www.mongodb.org/display/DOCS/Advanced+Queries
        """
        print("showHelp_nativefilter")
