"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from ui_generated import about

'''
Created on Feb 16, 2011

@author: martin
'''
from ui_generated.init_load import Ui_MainWindow
from PyQt4 import QtGui
from ui.WindowHandling import WindowUtils
from hatcore.DBConnector import MongoConnector
from hatcore.prefecences import prefs

class InitializationHandler():
    def __init__(self):
        window = QtGui.QMainWindow()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(window)
        
        # connect event handlers
        self.ui.actionAbout.triggered.connect(self.showAbout)
        self.ui.pushButton_connect.clicked.connect(self.onConnect)
        self.ui.pushButton_start.clicked.connect(self.onStart)
        WindowUtils.store(window,self)
        self.window = window
        #Check if there are stored values
        host = prefs.getPreference("db-host")
        port = prefs.getPreference("db-port")
        if host is not None:
            self.ui.lineEdit_dbhost.setText(str(host))
        if port is not None:
            self.ui.lineEdit_dbport.setText(str(port))
        window.show()

    
    def showAbout(self):
        WindowUtils.display(QtGui.QDialog(), about.Ui_Dialog())

        
    def showStatus(self, text):
        print("-> %s " % text)
        self.ui.statusbar.showMessage(text)
    
    def onConnect(self):
        host = str(self.ui.lineEdit_dbhost.text())
        port = int(self.ui.lineEdit_dbport.text())
        self.showStatus("Connecting to %s:%s" % (host, port))
        conn = MongoConnector(host, port)
        try:
            databases = conn.getDatabases()
            msg = "DB connection ok"
            if len(databases) <= 0 :
                msg = msg + ", but no databases found"
                self.ui.pushButton_start.setEnabled(False)
            else:
                (dbnames, displaynames) = databases
                self.ui.listWidget_sessions.addItems(displaynames)
                self.dbnames = dbnames
                self.ui.pushButton_start.setEnabled(True)
                self.conn = conn
        except Exception as e:
            msg = "Conn failure: %s "% (str(e)) 
            self.ui.pushButton_start.setEnabled(False)
        self.showStatus(msg)
        
    def onStart(self):
        prefs.setPreference("db-host",self.conn.host )
        prefs.setPreference("db-port",self.conn.port )
        prefs.save()
        
        
        row = self.ui.listWidget_sessions.selectedIndexes()[0].row()
        session = self.dbnames[row]
        
        self.showStatus("Using session: %s " % session)
        
        self.conn.useDatabase(session)

        from ui.TabledataWindow import TabledataMainwindow
        from ui.AggregationWindow import AggregationWindow
        
        #Display table data window
        WindowUtils.display(QtGui.QMainWindow(), TabledataMainwindow(self.conn))
        #Display aggregator window
        WindowUtils.display(QtGui.QWidget(), AggregationWindow(self.conn))
        self.window.close()