"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Feb 12, 2011

@author: martin
'''
from PyQt4 import QtGui, QtCore

from ui_generated import object_inspector
from ui.WindowHandling import WindowUtils

class ObjectInspectorHandler():
    """This class is responsible for the object inspector window."""
    
    def __init__(self, object):
        #Instantiate the UI class
        ui = object_inspector.Ui_Form()
        self.ui = ui
        # Get a window
        window = QtGui.QWidget()
        ui.setupUi(window)
        self.treeWidget = ui.treeWidget_objectinspector
        self.textEdit = ui.plainTextEdit
        #Connect signals/slots
     
        self.treeWidget.currentItemChanged.connect(self.__updateDisplay)

        #store self, which is a signal-receiver
        WindowUtils.store(window,self)
        self.setObject(object)
        window.show()
        
    def setObject(self, object):
        self.object = object
        self.treeWidget.clear()
        root = QtGui.QTreeWidgetItem(self.treeWidget,["<Object>","HTTP"])
        self.treeWidget.expandItem(root)
        self.__recurse(self.object, root)

    def __recurse(self, object, root):
        """Recurses through the given object and fills the qtreeview
        with data"""
        itemnum = 0
        for (k,v) in object.items():
            if type(v).__name__ <> 'dict':
                _v = str(v)
                if len(_v) > 50 :
                    _v = _v[:50]+ "..."
                QtGui.QTreeWidgetItem(root, [str(k), _v])
            else:
                child = QtGui.QTreeWidgetItem(root,[ str(k)])
                self.__recurse(v, child)
            itemnum += 1

 
    def __updateDisplay(self):
        """Updates the text-display showing the contents of the object"""
        self.textEdit.setPlainText("")
        if self.object is None: return
        
        item = self.treeWidget.currentItem()
        if item == None: return
        
        path = [str(item.text(0))]
        while item.parent() is not None:
            item = item.parent()
            path.append(str(item.text(0)))
            
        path.reverse()
        #Remove the root node, that is the object itself
        path = path[1:]
        
        obj = self.object
        for x in path:
            obj=obj[x]
            
        self.textEdit.setPlainText(str(obj))
