"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Feb 22, 2011

@author: martin
'''
from ui.WindowHandling import WindowUtils
from hatcore.prefecences import prefs


class ComboboxSettingsHandler():
    """
    This is a helper class which handles the saving of data into preferences. You throw in the components that are
    needed, plus the callback to the object which needs the settings, and this little helper calls getSaveDat() and 
    applySaveData() respectively when the user clicks for those actions
    """
    def __init__(self, comboBoxUIComp, loadButtonUIComp,saveButtonUIComp, saveNameUIComp, callback,name):
        #The name is used to distinguish the settings
        self.name = name
    
        self.settings = prefs.getPreference(self.name)
        if self.settings == None:
            self.settings = {}

        #These are the UI - components we need, 
        # A combobox to fill with values, and perform load from
        self.comboBox = comboBoxUIComp
        #Prefill the combo-box with saved settings
        for k in self.settings.keys():
            self.comboBox.addItem(str(k))
        # An input field where we read the name
        self.saveName = saveNameUIComp
        # a button which is clicked
        saveButtonUIComp.clicked.connect(self.onSave)
        #Also, the 'handlee', which we handle all this stuff for
        self.callback = callback
        
        loadButtonUIComp.clicked.connect(self.onLoad)
    
    def onLoad(self):
        name = str(self.comboBox.currentText())
        if len(name) == 0: return
        
        s = self.settings[name]
        self.callback.applySaveData(s, self.name)

    def onSave(self):
        """Saves the current 'setup' in a file"""
        name = str(self.saveName.text())
        if len(name) == 0:
            WindowUtils.alert("You need to supply a name for your setup")
            return
        self.settings[name] = self.callback.getSaveData(self.name)
        prefs.setPreference(self.name, self.settings)
        prefs.save()
        #Refill the combobox
        self.comboBox.clear()
        for k in self.settings.keys():
            self.comboBox.addItem(str(k))
        self.saveName.clear()
