"""
This file is part of the Hatkit Datafiddler, a tool for analyzing
MongoDB database content, particularly http traffic.
  
Copyright (C) 2011 Martin Holst Swende (hatkit@swende.se)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
'''
Created on Mar 24, 2010

@author: Martin Holst Swende 
'''
import time as timelib
def toupper(val):
    """Dummy example transform: makes the input into uppercase"""
    return str(val).upper()

def showparams(url=None, form=None):
    """
    Sorts parameters by keys. You can send in two dicts, and get the combined result. This makes it easier
    to show both form-data and url-data in the same column. Example

    variable v2: request.url
    variable v3: request.data
    column: sortparams(v2, v3)

    //Another version
    variable v1: request
    column: sortparams(form=v1.data,url=v1.url)

    
    @param url: Should be the request.url object (optional)
    @param form: Should be the request.data object (optional) 
    @return: uri-like string, e.g a=1&b=2&c=test 
    
    """
    retval = ""
    if url <> None:
        if "parameters" in url.keys():
            retval = dictToString(url['parameters'])
    
    if form <> None:
            if len(retval) > 0:
                retval += "&"
            retval += dictToString(form)
            
    #If error, just return the value
    return retval
def dictToString(val):
    """Flattens a dictionary to url-like string. 
    Example: dictToString({'x':'y', 'foo':'bar'}) => x=y&foo=bar
    """
    if type(val).__name__ == 'dict':
        keys = val.keys()
        keys.sort()
        p = []
        for key in keys:
            p.append("%s=%s" % (key, val[key]))
        return "&".join(p)
    #If error, just return the value
    return val

def paramstring(value):
    """Shows the parameterstring used.  Removes directory and file - info from the raw string
    @param value: either the request object, the request.url object or hte request.url.raw string"""
    rawstring = ""
    if type(value).__name__ == 'dict':
        o = value
        hdrs = o.keys()
        if "url" in hdrs:#This was a request object
            o = o["url"]
        if "raw" in o.keys(): #o should now be request.url
            rawstring = o["raw"]
    
    i = rawstring.find("?")
    if i > 0:
        return rawstring[i:]
    i = rawstring.find(";")
    if i > 0:
        return rawstring[i:]
    return ""
def getheader(o,header):
    """Gets a header value from either a response object,response.headers object, request object or request.headers object. 
    @return the header value or None"""
    if "headers" in o.keys():#This was a response object
        o = o["headers"]
    if header in o.keys():
        return o[header]
    return None

def secT():
    """It's full of stars"""
    return "**********"
def FQHost(request):
    """Creates a Fully Qualified hostname from a request object. 
    Example: 
    v0 = request
    FQHost(v0) => https://www.foobar.com:443
    """
    method = "http://"
    if request['ssl']:
        method= "https://"
    t = request['target'] # On the form: www.dn.se/62.119.189.4:80
    (domain, ip) = t.split("/")
    (foo, port) = t.split(":")
    fq = "%s%s:%s" % ( method, domain, port)
    return fq
    
def size(value):
    """Returns the content-length in human readable form. 
    @param value either the response object or the response.headers object
    """
    v = getheader(value, "Content-Length")
    if v != None : 
        s = int(v)
        if s < 1024:
            return "%d B" % s

        kb = float(s)/1024
        if kb < 1024:
            return "%.1f kB" % kb
        mb = kb / 1024
        return "%.2f MB" % mb
    return None
def date(time):
    """Returns the date on a short format (MMDD HH:MM:SS) 0301 15:30:25
    @param time The timestamp from the request (milliseconds)"""
    return timelib.strftime("%m%d %H:%M:%S",timelib.localtime(time/1000))

def longdate(time):
    """Returns the date on a longer format (YYYY MM DD HH:MM:SS)
    @param time The timestamp from the request"""
    return timelib.strftime("%Y %m %d %H:%M:%S",timelib.localtime(time/1000))

def time(time):
    """Returns the date on a really short format (HH:MM:SS) 15:30:30
    @param time The timestamp from the request"""
    return timelib.strftime("%H:%M:%S",timelib.localtime(time/1000))

def since(offset, time):
    """Returns the time since an offset. Usage : 
    since(12301230, request.time) => 50s or 1m23s
    @param offset the timestamp to count from
    @param time The timestamp from the request
    @return string"""
    delta = time - offset
    if ( delta < 1000): # Milliseconds
        return "%dms" % delta
    ms = delta % 1000
    delta = (delta - ms) / 1000
    s = delta % 60
    delta = (delta - s) / 60
    if ( delta == 0): # seconds
        return "%ds%dms" % (s,ms) 
    m = delta % 60
    h = (delta - m) / 60
    if (h == 0):
        return "%dm%ds%dms" % (m,s,ms)
    return "%dh%dm%ds%dms" % (h,m,s,ms)
def pfilter_only(parameter, parameters):
    """ Paramfilter : Only
    Filters the parameters and shows only one parameter
    @param parameter String name of the parameter to show
    @param parameters : a dict with parameters (e.g request.url.parameters or request.data)
    @return dictionary or None
    """
    if parameters == None:
        return None
    if parameter in parameters.keys():
        return {parameter:parameters[parameter]}
    return None

def pfilter_remove(remove, params):
    """ Paramfilter : Remove
    Filters the parameters and hides selected parameter
    @param remove String name of the parameter to filter
    @param parameters : request.url.parameters
    @return dictionary
    """
    return "not impl"

def tfilter_only(tag, tags):
    """ Tagfilter : Only
    """
    return "not impl"

def tfilter_remove(tag, tags):
    """Tagfilter : Remove
    """
    return "not impl"

def browser(request):
    """
    Checks what browser issued the request based on UA-string. 
    @param request - request object
    @return a short browser indicator, e.g "IE 6.0" or "FF 3.5" 
    """
    #FF on linux : 
    #Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.1.8) Gecko/20100214 Ubuntu/9.10 (karmic) Firefox/3.5.8
    #IE on windows : 
    #User-Agent: Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; .NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)
    val =""
    agent = getheader("User-Agent")
    if agent != None:
        i = agent.find("MSIE")
        if i > 0:
            val = agent[i:i+8] 
        else: 
            b = agent.split(" ")[-1]
            val = b.replace("Firefox", "FF")
        
    return val

def setcookie(response):
    """
    Shows any setcookie-commands returned from the server
    @param response : either response-object or response.headers
    """
    return getheader(response, "Set-Cookie")

def cookies(request):
    """Shows cookies
    @param request: either request object or request.headers
    """
    return getheader(request,"Cookie")

class Transformer(object):
    '''
    This class provides the running of transformer functions and generating help-texts
    '''
    def apply(self, transformation, variables):
        #print("Transformer.apply("+transformation+" , "+str(variables)+")")
        value = None
        if (transformation != None and len(transformation) > 0):
            try:
                value = eval(transformation, globals(), variables)
            except Exception as e:
                value = "%s:%s" % (type(e).__name__, str(e))
        return value
    
    def getHelp(self):
        g = globals()
        itemdoc = ""
        for name, func in g.items():
            if type(func).__name__ == 'function':
                argcount = func.func_code.co_argcount
                varnames = func.func_code.co_varnames[:argcount]
                doc = func.__doc__.replace("\n","<br/>")
                print "%s(%s):\r\n%s" % (name, ",".join(varnames), doc)
                itemdoc = itemdoc + "<p><b>%s(%s)</b></p><p>%s</p><hr/>" % (name, ",".join(varnames), doc)
        html = "<html><body>%s</body></html>" % itemdoc
        return html
transformer = Transformer()

if __name__ == '__main__':
    transformer.getHelp()
    value = transformer.apply("toupper(v1)", {"v1":"test"})
    print value
